@@ -1,14 +1,13 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2008-2011 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2008 -- leonerd@leonerd.org.uk
 
 package IO::Async::Sequencer;
 
 use strict;
-use warnings;
 
-our $VERSION = '0.51_002';
+our $VERSION = '0.14';
 
 use base qw( IO::Async::Stream );
 
@@ -22,8 +21,8 @@ C<IO::Async::Sequencer> - handle a serial pipeline of requests / responses (EXPE
 
 When used as a client:
 
- use IO::Async::Loop;
- my $loop = IO::Async::Loop->new;
+ use IO::Async::Loop::IO_Poll;
+ my $loop = IO::Async::Loop::IO_Poll->new();
 
  my $sock = ...
 
@@ -31,7 +30,7 @@ When used as a client:
     handle => $sock,
 
     on_read => sub {
-       my ( $self, $buffref, $eof ) = @_;
+       my ( $self, $buffref, $closed ) = @_;
 
        return 0 unless $$buffref =~ s/^(.*)\n//;
        my $line = $1;
@@ -64,7 +63,7 @@ When used as a server:
     handle => $sock,
 
     on_read => sub {
-       my ( $self, $buffref, $eof ) = @_;
+       my ( $self, $buffref, $closed ) = @_;
 
        return 0 unless $$buffref =~ s/^(.*)\n//;
        my $line = $1;
@@ -114,7 +113,7 @@ Objects in this class can operate in any of three ways:
 =item * A pure client
 
 The object is asked to send requests by the containing code and invokes
-response handling code when responses arrive.
+response callbacks when responses arrive.
 
 =item * A pure server
 
@@ -129,54 +128,62 @@ responding to those of its peer connection.
 =back
 
 The exact mode of operation of any object is not declared explicitly, but
-instead is an artefact of the set of callbacks provided to the constructor or
-methods defined by the subclass. Certain callbacks or methods only make sense
-for one mode or the other.
-
-The various operations required can each be provided as callback functions
-given in keys to the constructor, or as object methods on a subclass of this
-class. Keys passed to the constructor will take precidence over defined
-methods.
+instead is an artefact of the set of callbacks provided to the constructor,
+and the methods called on the object. Certain callbacks or methods only make
+sense for one mode or the other.
 
 As it is still EXPERIMENTAL, any details of this class are liable to change in
 future versions. It shouldn't yet be relied upon as a stable interface.
 
 =cut
 
-=head1 PARAMETERS
+=head1 CONSTRUCTOR
 
-The following named parameters may be passed to C<new> or C<configure>:
+=cut
+
+=head2 $sequencer = IO::Async::Sequencer->new( %params )
+
+This function returns a new instance of a C<IO::Async::Sequencer> object. The
+C<%params> hash takes the following keys:
 
 =over 8
 
+=item read_handle => IO
+
+=item write_handle => IO
+
+=item handle => IO
+
+As for L<IO::Async::Stream>.
+
 =item on_read => CODE
 
 As for L<IO::Async::Stream>. The code here should invoke the
-C<incoming_request> or C<incoming_response> methods when appropriate,
+C<incoming_request()> or C<incoming_response()> methods when appropriate,
 after having parsed the incoming stream. See the SYNOPSIS or EXAMPLES sections
 for more detail.
 
 Each request can optionally provide its own handler for reading its response,
-using the C<on_read> key to the C<request> method. The handler provided to
+using the C<on_read> key to the C<request()> method. The handler provided to
 the constructor is only used if this is not provided.
 
 =item on_request => CODE
 
-A callback that is invoked when the C<incoming_request> method is called
+A callback that is invoked when the C<incoming_request()> method is called
 (i.e. when operating in server mode). It is passed the request, and a token to
 identify it when sending a response.
 
  $on_request->( $self, $token, $request );
 
 The token should be considered as an opaque value - passed into the
-C<respond> method when a response is ready, but not otherwise used or
+C<respond()> method when a response is ready, but not otherwise used or
 modified.
 
 =item marshall_request => CODE
 
 =item marshall_response => CODE
 
-Callbacks that the C<request> or C<respond> methods will use,
+Callbacks that the C<request()> or C<respond()> methods will use,
 respectively, to stream a request or response object into a string of bytes to
 write to the underlying file handle.
 
@@ -186,48 +193,30 @@ write to the underlying file handle.
 
 These are used respectively by the client and server modes.
 
-=item pipeline => BOOL
-
-Optional. Controls whether requests will be pipelined; that is, all requests
-will be sent by the client before responses are received. If this option is
-disabled, only the first request will be sent. Other requests will be queued
-internally, and each will be sent when the response to the previous has been
-received. Defaults enabled; supply a defined but false value to disable.
-
 =back
 
 =cut
 
-sub _init
+sub new
 {
-   my $self = shift;
-   my ( $params ) = @_;
+   my $class = shift;
+   my %params = @_;
 
-   $self->SUPER::_init( @_ );
+   defined $params{handle} or
+      ( defined $params{read_handle} and defined $params{write_handle} ) or
+      croak "Sequencer requires both a reading and a writing handle"; # TODO: this message sucks
 
-   $self->{pipeline} = 1; # default on
+   my $on_read = delete $params{on_read};
+   ref $on_read eq "CODE" or croak "Expected 'on_read' as a CODE reference";
 
-   # Queue to use in server mode - stores pending responses to be sent
-   $self->{server_queue} = []; # element is: $streamed_response
+   my $self = $class->SUPER::new(
+      %params,
 
-   # Queue to use in client mode - stores pending on_response handlers to be called
-   $self->{client_queue} = []; # element is: [ $on_response, $delegated_on_read ]
-
-   my $on_read = delete $params->{on_read} || $self->can( "on_read" );
-
-   # Since our ->configure has banned 'on_read', we need to call SUPER one from here
-   $self->SUPER::configure(
       on_read => sub {
-         my ( $self, $buffref, $eof ) = @_;
+         my ( $self, $buffref, $closed ) = @_;
 
          my $front = $self->{client_queue}[0];
 
-         if( !$self->{pipeline} and $front and defined $front->[2] ) {
-            # Next request needs sending
-            $self->write( $front->[2] );
-            undef $front->[2];
-         }
-
          if( $front and $front->[1] ) {
             # Delegate to the one provided by the request
             my $delegated_on_read = $front->[1];
@@ -242,26 +231,19 @@ sub _init
          goto &$on_read;
       },
    );
-}
 
-sub configure
-{
-   my $self = shift;
-   my %params = @_;
+   $self->{marshall_request}  = $params{marshall_request};
+   $self->{marshall_response} = $params{marshall_response};
 
-   if( exists $params{on_read} ) {
-      croak "Cannot modify 'on_read' of a " . __PACKAGE__;
-   }
+   $self->{on_request} = $params{on_request};
 
-   foreach (qw( marshall_request marshall_response on_request )) {
-      $self->{$_} = delete $params{$_} if exists $params{$_};
-   }
+   # Queue to use in server mode - stores pending responses to be sent
+   $self->{server_queue} = []; # element is: $streamed_response
 
-   if( exists $params{pipeline} ) {
-      $self->{pipeline} = !!delete $params{pipeline};
-   }
+   # Queue to use in client mode - stores pending on_response handlers to be called
+   $self->{client_queue} = []; # element is: [ $on_response, $delegated_on_read ]
 
-   $self->SUPER::configure( %params );
+   return $self;
 }
 
 =head1 SUBCLASS METHODS
@@ -274,11 +256,11 @@ subclass, rather than the containing application code.
 
 =head2 $sequencer->incoming_request( $request )
 
-To be called from C<on_read>.
+To be called from the C<on_read> callback.
 
 This method informs the sequencer that a new request has arrived. It will
-invoke C<on_request>, passing in a token to identify the request for stream
-ordering purposes, and the request itself.
+invoke the C<on_request> callback, passing in a token to identify the
+request, for stream ordering purposes, and the request itself.
 
 =cut
 
@@ -287,16 +269,10 @@ sub incoming_request
    my $self = shift;
    my ( $request ) = @_;
 
-   my $on_request = $self->{on_request}
-                    || $self->can( "on_request" );
-
-   defined $on_request or
-      croak "Cannot process incoming request without an 'on_request'";
-
    push @{ $self->{server_queue} }, undef;
    my $token = \$self->{server_queue}[-1];
 
-   $on_request->( $self, $token, $request );
+   $self->{on_request}->( $self, $token, $request );
 }
 
 sub _flush_server_queue
@@ -311,11 +287,11 @@ sub _flush_server_queue
 
 =head2 $sequencer->incoming_response( $response )
 
-To be called from C<on_read>.
+To be called from the C<on_read> callback.
 
-This method informs the sequencer that a response has arrived. It will invoke
-C<on_response> that had been passed to the C<request> method that sent the
-original request.
+This method informs the sequencer that a response has arrived. It will
+invoke the C<on_response> callback that had been passed to the C<request>
+method that sent the original request.
 
 =cut
 
@@ -324,21 +300,11 @@ sub incoming_response
    my $self = shift;
    my ( $response ) = @_;
 
-   my $client_queue = $self->{client_queue};
-
-   my $cq = shift @$client_queue;
+   my $cq = shift @{ $self->{client_queue} };
    my $on_response = $cq->[0];
 
    defined $on_response or croak "Cannot 'incoming_response' without a stored 'on_response' handler";
 
-   my $front = $self->{client_queue}[0];
-
-   # Send the next request if there's one queued
-   if( !$self->{pipeline} and $front and defined $front->[2] ) {
-      $self->write( $front->[2] );
-      undef $front->[2];
-   }
-
    $on_response->( $response );
 }
 
@@ -355,7 +321,7 @@ Called in client mode, this method sends a request upstream, and awaits a
 response to it. Can be called in one of two ways; either giving a specific
 C<on_read> handler to be used when the response to this request is expected,
 or by providing an C<on_response> handler for when the default handler invokes
-C<incoming_response>.
+C<incoming_response()>.
 
 The C<%params> hash takes the following arguments:
 
@@ -363,12 +329,12 @@ The C<%params> hash takes the following arguments:
 
 =item request => SCALAR
 
-The request value to pass to C<marshall_request>.
+The request value to pass to the C<marshall_request> callback.
 
 =item on_response => CODE
 
-A continuation to invoke when a response to this request arrives from the
-upstream server. It will be invoked as
+A callback to invoke when a response to this request arrives from the upstream
+server. It will be invoked as
 
  $on_response->( $response );
 
@@ -378,12 +344,11 @@ A callback to use to parse the incoming stream while the response to this
 particular request is expected. It will be invoked the same as for
 C<IO::Async::Stream>; i.e.
 
- $on_read->( $self, $buffref, $eof )
+ $on_read->( $self, $buffref, $closed )
 
 This handler should return C<undef> when it has finished handling the
 response, so that the next one queued can be invoked (or the default if none
-exists). It MUST NOT call C<incoming_response>. Instead, the code should
-directly implement the behaviour for receipt of a response.
+exists). It MUST NOT call C<incoming_response()>.
 
 =back
 
@@ -400,11 +365,8 @@ sub request
 
    my $request = $params{request};
 
-   my $marshall_request = $self->{marshall_request}
-                           || $self->can( "marshall_request" );
-
-   defined $marshall_request or
-      croak "Cannot send request without a 'marshall_request'";
+   defined( $self->{marshall_request} ) or
+      croak "Cannot send request without a 'marshall_request' callback";
 
    my $on_response = $params{on_response};
    my $on_read     = $params{on_read};
@@ -415,26 +377,16 @@ sub request
    defined $on_response or defined $on_read or
       croak "Need one of 'on_response' or 'on_read'";
 
-   my $client_queue = $self->{client_queue};
+   $self->write( $self->{marshall_request}->( $self, $request ) );
 
-   my $request_encoded = $marshall_request->( $self, $request );
-
-   if( $self->{pipeline} or not @$client_queue ) {
-      # Clear to send
-      $self->write( $request_encoded );
-      push @{ $self->{client_queue} }, [ $on_response, $on_read, undef ];
-   }
-   else {
-      # Have to wait
-      push @{ $self->{client_queue} }, [ $on_response, $on_read, $request_encoded ];
-   }
+   push @{ $self->{client_queue} }, [ $on_response, $on_read ];
 }
 
 =head2 $sequencer->respond( $token, $response )
 
-Called in server mode, usually at the end of C<on_request>, or some
-continuation created within it, this method sends a response back downstream
-to a client that had earlier requested it.
+Called in server mode, usually at the end of the C<on_request> callback, or
+some continuation created within it, this method sends a response back
+downstream to a client that had earlier requested it.
 
 =over 8
 
@@ -445,7 +397,7 @@ sent in the right order.
 
 =item $response
 
-The response value to pass to C<marshall_response>.
+The response value to pass to the C<marshall_response> callback.
 
 =back
 
@@ -456,19 +408,21 @@ sub respond
    my $self = shift;
    my ( $token, $response ) = @_;
 
-   my $marshall_response = $self->{marshall_response} 
-                            || $self->can( "marshall_response" );
+   defined( $self->{marshall_response} ) or
+      croak "Cannot send response without a 'marshall_response' callback";
 
-   defined $marshall_response or
-      croak "Cannot send response without a 'marshall_response'";
-
-   my $response_stream = $marshall_response->( $self, $response );
+   my $response_stream = $self->{marshall_response}->( $self, $response );
 
    $$token = $response_stream;
 
    $self->_flush_server_queue;
 }
 
+# Keep perl happy; keep Britain tidy
+1;
+
+__END__
+
 =head1 EXAMPLES
 
 =head2 A simple line-based server
@@ -476,53 +430,46 @@ sub respond
 The following sequencer implements a simple server which takes and responds
 with CRLF-delimited lines.
 
- package LineSequencer;
-
- use base qw( IO::Async::Sequencer );
-
  my $CRLF = "\x0d\x0a"; # More portable than \r\n
 
- sub on_read {
-    my ( $self, $buffref, $eof ) = @_;
+ my $sequencer = IO::Async::Sequencer->new(
+    ...
 
-    while( $buffref =~ s/^(.*)$CRLF// ) {
-       $self->incoming_request( $1 );
+    on_read => sub {
+       my ( $self, $buffref, $closed ) = @_;
+
+       $buffref =~ s/^(.*)$CRLF// and
+          $self->incoming_request( $1 ), return 1;
+
+       return 0;
     }
 
-    return 0;
- }
-
- sub marshall_response {
-    my ( $self, $response ) = @_;
-    return $response . $CRLF;
- }
-
- 1;
-
-The server could then be used, for example, as a simple echo server that
-replies whatever the client said, in uppercase. This would be done using an
-C<on_request> like the following.
-
- my $linesequencer = LineSequencer->new(
-    handle => ...
-
-    on_request => sub {
-       my ( $self, $token, $request ) = @_;
-       $self->respond( $token, uc $request );
+    marshall_response => sub {
+       my ( $self, $response ) = @_;
+       return $response . $CRLF;
     }
  );
 
+The server could then be used, for example, as a simple echo server that
+replies whatever the client said, in uppercase. This would be done using an
+C<on_request> callback like the following.
+
+ on_request => sub {
+    my ( $self, $token, $request ) = @_;
+    $self->respond( $token, uc $request );
+ }
+
 It is likely, however, that any real use of the server in a non-trivial way
-would perform much more work than this, and only call C<< $self->respond >>
+would perform much more work than this, and only call C<< $self->respond() >>
 in an eventual continuation at the end of performing its work. The C<$token>
 is used to identify the request that the response responds to, so that it can
 be sent in the correct order.
 
 =head2 Per-request C<on_read> handler
 
-If an C<on_read> handler is provided to the C<request> method in client
+If an C<on_read> handler is provided to the C<request()> method in client
 mode, then that handler will be used when the response to that request is
-expected to arrive. This will be used instead of the C<incoming_response>
+expected to arrive. This will be used instead of the C<incoming_response()>
 method and the C<on_response> handler. If every request provides its own
 handler, then the one in the constructor would only be used for unrequested
 input from the server - perhaps to generate an error condition of some kind.
@@ -531,7 +478,7 @@ input from the server - perhaps to generate an error condition of some kind.
     ...
 
     on_read => sub {
-       my ( $self, $buffref, $eof ) = @_;
+       my ( $self, $buffref, $closed ) = @_;
 
        print STDERR "Spurious input: $$buffref\n";
        $self->close;
@@ -547,7 +494,7 @@ input from the server - perhaps to generate an error condition of some kind.
  $sequencer->request(
     request => "some key",
     on_read => sub {
-       my ( $self, $buffref, $eof ) = @_;
+       my ( $self, $buffref, $closed ) = @_;
 
        return 0 unless $$buffref =~ s/^(.*)$CRLF//;
        my $line = $1;
@@ -564,9 +511,15 @@ input from the server - perhaps to generate an error condition of some kind.
 
 =item *
 
-Some consideration of streaming errors. How does the C<on_read> signal to the
-containing object that a stream error has occured? Is it fatal?  Can
-resynchronisation be attempted later?
+Some consideration of streaming errors. How does the C<on_read> callback
+signal to the containing object that a stream error has occured? Is it fatal?
+Can resynchronisation be attempted later?
+
+=item *
+
+More support for actually being a subclass, with methods for
+C<marshall_request> and C<marshall_response> in the way that C<on_read> can
+be.
 
 =item *
 
@@ -578,8 +531,4 @@ were sent.
 
 =head1 AUTHOR
 
-Paul Evans <leonerd@leonerd.org.uk>
-
-=cut
-
-0x55AA;
+Paul Evans E<lt>leonerd@leonerd.org.ukE<gt>
